/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Iterator
*  SCOPE :              Iterator
*  FILE :               iter.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __iter_HXX
#define __iter_HXX

/* IMPORTS *******************************************************************/

#ifndef __col_HXX
#include "col.hxx"
#endif

#ifndef __listx_HXX
#include "listx.hxx"
#endif


/*****************************************************************************/

class GenIterator {

protected :

  // Current indice of the scan
  int _currPos ;

  // First and last indice of the scan
  int _beginPos, _endPos ;

  // Collection scan
  Collection* _scanCol ;

  // Current leaf scan
  ColLeaf* _currLeaf ;

  // First and last indice of the current leaf
  int _localPos, _localEnd ;

  public :

  GenIterator() {} ;
  GenIterator(Collection* c, int beg, int end) ;

  // Return TRUE if there is more elements to scan
  int more() { return _currPos < _endPos ; }

  // Return current indice of the scan
  int where() { return _currPos ; }

  // Reset the scan (could change edges)
  void reset(int begin = 0, int end = -1) ;

  // move current element scan forward
  void next() ;

  // move current element scan backward
  void prev() ;

  // move scan directly to position 'pos'
  void go(int pos) ;  
} ;


#include "iter_tmpl.hxx"


class Iterator<int> : public GenIterator {

  public :

  Iterator() {} ;
  Iterator(const LocalListX<int>& l, int begin=0, int end=-1) 
  : GenIterator(l.col(), begin, end) {}

  int& get() { return *(int*)_currLeaf->tab(_localPos)->data() ; }
} ;


class Iterator<double> : public GenIterator {

  public :

  Iterator() {} ;
  Iterator(const LocalListX<double>& l, int begin=0, int end=-1) 
  : GenIterator(l.col(), begin, end) {}

  double& get() { return *(double*)_currLeaf->tab(_localPos)->data() ; }
} ;

class Iterator<StringX> : public GenIterator {

  public :

  Iterator() {} ;
  Iterator(const LocalListX<StringX>& l, int begin=0, int end=-1) 
  : GenIterator(l.col(), begin, end) {}

  StringX& get() { return *(StringX*)_currLeaf->tab(_localPos)->data() ; }
} ;


#define yoo_declare_Iterator(ELEM) \
     template class Iterator<ELEM> ;

#endif
